#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

#define movementsensor A5

#define solarpanel     A1 // 0 - Max 1023
#define max_solarpanel 900
#define min_solarpanel 1
#define poll_solarpanel 40000 // millis

#define max_neopixel        255
#define max_neopixel_dark   64
#define min_neopixel        16

#define neopixels      2
Adafruit_NeoPixel strip = Adafruit_NeoPixel(35, neopixels, NEO_GRB + NEO_KHZ800); // Only the number we need to address
Adafruit_NeoPixel reset = Adafruit_NeoPixel(40, neopixels, NEO_GRB + NEO_KHZ800); // Full array

// IMPORTANT: To reduce NeoPixel burnout risk, add 1000 uF capacitor across
// pixel power leads, add 300 - 500 Ohm resistor on first pixel's data input
// and minimize distance between Arduino and first pixel.  Avoid connecting
// on a live circuit...if you must, connect GND first.

unsigned long movement;
unsigned long light;
byte          theme        = 0;
bool          forceupdate  = true;

unsigned long currentMillis = millis();
unsigned long lastMillis    = currentMillis;

int intensity;

int r;
int g;
int b;


void setup() {
  // This is for Trinket 5V 16MHz, you can remove these three lines if you are not using a Trinket
  #if defined (__AVR_ATtiny85__)
    if (F_CPU == 16000000) clock_prescale_set(clock_div_1);
  #endif   // End of trinket special code

  Serial.begin(9600);

  pinMode(movementsensor, INPUT);
  pinMode(solarpanel, INPUT);

  randomSeed(analogRead(solarpanel));

  delay(1000);
  reset.begin();
  reset.show(); // Initialize all pixels to 'off'

  delay(1000);
  strip.begin();
  strip.show(); // Initialize all pixels to 'off' for only the number of pixels we are using.
}

void loop() {

  currentMillis = millis();

  // Update every N seconds or when forced
  if ( ( currentMillis > lastMillis && currentMillis - lastMillis > poll_solarpanel ) || forceupdate) {
    UpdateLightIntensity();
  }

  // Execute a theme on movement
  if ( digitalRead(movementsensor) ) {
    ExecuteTheme();
  }
}

void ExecuteTheme() {
  switch (theme) {
    case 0:
      TurnAllOff(false, 50);
      Serial.println(F("4 color theater chase"));
      for ( int i=0 ; i < 4 ; i++) {
        r=random(0,255);
        g=random(0,255);
        b=random(0,255);

        theaterChase(strip.Color(r, g, b), 50);
      }  
      TurnAllOff(true, 50);
      break;
    case 1: 
      TurnAllOff(false, 50);
      Serial.println(F("Theater chase rainbow"));
      theaterChaseRainbow(50);
      TurnAllOff(true, 50);
      break;
    case 2:
      TurnAllOff(false, 50);
      Serial.println(F("Rainbow cycle"));
      rainbowCycle(10);
      TurnAllOff(true, 50);
      break;
    default:
      TurnAllOff(false, 50);
      Serial.println(F("Rainbow"));
      rainbow(100);
      TurnAllOff(true, 50);
      break;
  }
  theme++;
  if ( theme > 3 ) theme = 0;

  forceupdate=true;
}

void UpdateLightIntensity() {    
      light    = constrain(analogRead(solarpanel),min_solarpanel,max_solarpanel);
  
      if  ( light > 768 ) {       // Intensity high!
          r = map(light,768,max_solarpanel,min_neopixel,max_neopixel);
          g = map(light,768,max_solarpanel,max_neopixel,min_neopixel);
          b = 0;
      } else if ( light > 512 ) { // Intensity medium!
          r = 0;
          g = map(light,512,768,min_neopixel,max_neopixel);
          b = map(light,512,768,max_neopixel,min_neopixel);
      } else if ( light > 256 ) { // Intensity low!
          r = map(light,256,512,max_neopixel,min_neopixel);
          g = map(light,256,512,max_neopixel,min_neopixel);
          b = max_neopixel;
      } else {                    // It's dark!
          intensity = map(light, min_solarpanel, max_solarpanel, min_neopixel, max_neopixel_dark);
          r = light;
          g = light;
          b = light;        
      }
    
      Serial.println(String("Color ") + F("R:") + String(r) + F(" G:") + String(g) + F(" B:") + String(b));
      colorWipe(strip.Color(r,g,b), 50);

      lastMillis = currentMillis;
      forceupdate=false;
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, c);
    strip.show();    
    delay(wait);
  }
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256; j++) {
    for(i=0; i<strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel((i+j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< strip.numPixels(); i++) {
      strip.setPixelColor(i, Wheel(((i * 256 / strip.numPixels()) + j) & 255));
    }
    strip.show();
    delay(wait);
  }
}

void TurnAllOff(bool walkdown, uint8_t wait) {

  if ( walkdown ) {
    for (int i=0; i < strip.numPixels(); i++) { 
      strip.setPixelColor(i, 0);        //turn every pixel off
      strip.show();
      delay(wait);
    }
  } else {
    for (int i=strip.numPixels(); i >= 0; i--) {
      strip.setPixelColor(i, 0);        //turn every pixel off 
      strip.show();
      delay(wait);
    }
  }
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) {
  int step=4;
  
  for (int j=0; j<10; j++) {  //do 10 cycles of chasing
    for (int q=0; q < step; q++) {
      for (int i=0; i < strip.numPixels(); i+=step) {
        strip.setPixelColor(i+q, c);    //turn every nth pixel on
      }
      strip.show();

      delay(wait);

      for (int i=0; i < strip.numPixels(); i+=step) {
        strip.setPixelColor(i+q, 0);        //turn every nth pixel off
      }
    }
  }
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
  for (int j=0; j < 256; j++) {     // cycle all 256 colors in the wheel
    for (int q=0; q < 3; q++) {
      for (int i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, Wheel( (i+j) % 255));    //turn every third pixel on
      }
      strip.show();

      delay(wait);

      for (int i=0; i < strip.numPixels(); i=i+3) {
        strip.setPixelColor(i+q, 0);        //turn every third pixel off
      }
    }
  }
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;

  if(WheelPos < 85) {
    return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}
